from enum import auto
from typing import Any, Dict, Optional

from ...shared import Link
from ...utils import AutoValueEnum


class PaginationMethods(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#paging-extension
    """

    GET = auto()
    POST = auto()


class PaginationRelations(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#paging-extension
    """

    next = auto()
    previous = auto()


class PaginationLink(Link):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#paging-extension
    """

    rel: PaginationRelations
    method: PaginationMethods
    body: Optional[Dict[Any, Any]]
    merge: bool = False



from typing import List

from ..collection import Collection
from ..item import ItemCollection


class SingleFileStac(ItemCollection):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/single-file-stac#single-file-stac-specification
    """

    collections: List[Collection]

    def to_dict(self):
        return self.dict(by_alias=True, exclude_unset=True)



from enum import Enum
from typing import Dict, Optional, Type

from pydantic import BaseModel


class AutoValueEnum(Enum):
    def _generate_next_value_(name, start, count, last_values):
        return name


def decompose_model(model: Type[BaseModel]) -> Dict:
    """Decompose a pydantic model into a dictionary of model fields"""
    fields = {}
    for (k, v) in model.__fields__.items():
        type = v.outer_type_
        if not v.required:
            type = Optional[type]
        fields[k] = (type, v.field_info)
    return fields



from typing import Optional

from pydantic import BaseModel, Field

from ..shared import NumType


class ViewExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/view#item-fields
    """

    off_nadir: Optional[NumType] = Field(None, alias="view:off_nadir")
    incidence_angle: Optional[NumType] = Field(None, alias="view:incidence_angle")
    azimuth: Optional[NumType] = Field(None, alias="view:azimuth")
    sun_azimuth: Optional[NumType] = Field(None, alias="view:sun_azimuth")
    sun_elevation: Optional[NumType] = Field(None, alias="view:sun_elevation")

    class Config:
        allow_population_by_field_name = True



from enum import auto
from typing import Optional

from pydantic import BaseModel, Field

from ..utils import AutoValueEnum


class OrbitStates(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/sat#item-fields
    """

    ascending = auto()
    descending = auto()
    geostationary = auto()


class SatelliteExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/sat#satellite-extension-specification
    """

    orbit_state: Optional[OrbitStates] = Field(None, alias="sat:orbit_state")
    relative_orbit: Optional[int] = Field(None, alias="sat:relative_orbit")

    class Config:
        use_enum_values = True
        allow_population_by_field_name = True



from urllib.parse import urlparse

from .collection_assets import CollectionAssetExtension
from .datacube import DatacubeExtension
from .eo import ElectroOpticalExtension
from .item_assets import ItemAssetExtension
from .label import LabelExtension
from .pc import PointCloudExtension
from .projection import ProjectionExtension
from .sar import SARExtension
from .sat import SatelliteExtension
from .sci import ScientificExtension
from .timestamps import TimestampsExtension
from .version import VersionExtension
from .view import ViewExtension


class Extensions:
    collection_assets = CollectionAssetExtension
    datacube = DatacubeExtension
    eo = ElectroOpticalExtension
    item_assets = ItemAssetExtension
    label = LabelExtension
    pointcloud = PointCloudExtension
    projection = ProjectionExtension
    sar = SARExtension
    sat = SatelliteExtension
    scientific = ScientificExtension
    timestamps = TimestampsExtension
    version = VersionExtension
    view = ViewExtension

    aliases = {}

    @classmethod
    def register(cls, k, v, alias=None):
        setattr(cls, k, v)

        if alias:
            cls.aliases[alias] = k

    @classmethod
    def get(cls, k):
        if not urlparse(k).scheme:
            k = k.replace("-", "_")
        try:
            return getattr(cls, k)
        except AttributeError:
            try:
                return getattr(cls, cls.aliases[k])
            except KeyError:
                raise AttributeError(f"Invalid extension name or alias: {k}")



from enum import auto
from typing import List, Optional

from pydantic import BaseModel, Field

from ..shared import NumType
from ..utils import AutoValueEnum


class ChannelTypes(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/pointcloud#schema-object
    """

    floating = auto()
    unsigned = auto()
    signed = auto()


class SchemaObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/pointcloud#schema-object
    """

    name: str
    size: int
    type: ChannelTypes

    class Config:
        use_enum_values = True


class StatsObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/pointcloud#stats-object
    """

    average: Optional[NumType]
    count: Optional[int]
    maximum: Optional[NumType]
    minimum: Optional[NumType]
    name: Optional[str]
    position: Optional[int]
    stddev: Optional[NumType]
    variance: Optional[NumType]


class PointCloudExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/pointcloud#point-cloud-extension-specification
    """

    count: int = Field(..., alias="pc:count")
    type: str = Field(..., alias="pc:type")
    encoding: str = Field(..., alias="pc:encoding")
    schemas: List[SchemaObject] = Field(..., alias="pc:schemas")
    density: Optional[int] = Field(None, alias="pc:density")
    statistics: Optional[List[StatsObject]] = Field(None, alias="pc:statistics")

    class Config:
        use_enum_values = True
        allow_population_by_field_name = True



from enum import auto
from typing import Dict, List, Optional, Tuple, Union

from pydantic import BaseModel, Field

from ..shared import NumType
from ..utils import AutoValueEnum


class HorizontalAxis(str, AutoValueEnum):
    x = auto()
    y = auto()


class DimensionObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/datacube#additional-dimension-object
    """

    type: str
    description: Optional[str]
    extent: Optional[Tuple[NumType, NumType]]
    values: Optional[List[Union[NumType, str]]]
    step: Optional[Union[NumType, str]]
    unit: Optional[str]
    reference_system: Optional[int]
    axis: Optional[str]


class HorizontalSpatialDimension(DimensionObject):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/datacube#horizontal-spatial-dimension-object
    """

    type: str = Field("spatial", const=True)
    axis: HorizontalAxis
    extent: List[NumType]

    class Config:
        use_enum_values = True


class VerticalSpatialDimension(HorizontalSpatialDimension):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/datacube#vertical-spatial-dimension-object
    """

    axis: str = Field("z", const=True)


class TemporalDimension(DimensionObject):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/datacube#temporal-dimension-object
    """

    type: str = Field("temporal", const=True)
    extent: Tuple[NumType, NumType]


class DatacubeExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/datacube#data-cube-extension-specification
    """

    dimensions: Dict[
        str,
        Union[
            HorizontalSpatialDimension,
            VerticalSpatialDimension,
            TemporalDimension,
            DimensionObject,
        ],
    ] = Field(..., alias="cube:dimensions")

    class Config:
        use_enum_values = True
        allow_population_by_field_name = True



from typing import List, Optional

from pydantic import BaseModel

from ..shared import Asset


class CollectionAsset(Asset):
    href: Optional[str]


class ItemAssetExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/item-assets
    """

    item_assets: List[CollectionAsset]



from typing import Optional

from pydantic import BaseModel


class VersionExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/version#item-properties-and-collection-fields
    """

    version: str
    deprecated: Optional[bool]



from enum import auto
from typing import List, Optional

from pydantic import BaseModel, Field

from ..utils import AutoValueEnum


class CommonBandNames(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/eo#common-band-names
    """

    coastal = auto()
    blue = auto()
    green = auto()
    red = auto()
    yellow = auto()
    pan = auto()
    rededge = auto()
    nir = auto()
    nir08 = auto()
    nir09 = auto()
    cirrus = auto()
    swir16 = auto()
    swir22 = auto()
    lwir = auto()
    lwir11 = auto()
    lwir12 = auto()


class BandObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/eo#band-object
    """

    name: Optional[str]
    common_name: Optional[CommonBandNames]
    center_wavelength: Optional[float]
    full_width_half_max: Optional[float]
    description: Optional[str]


class ElectroOpticalExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/eo#electro-optical-extension-specification
    """

    bands: Optional[List[BandObject]] = Field(None, alias="eo:bands")
    cloud_cover: Optional[float] = Field(None, alias="eo:cloud_cover")

    class Config:
        use_enum_values = True
        allow_population_by_field_name = True



from typing import List, Optional, Union

from pydantic import BaseModel, Field

from ..shared import ExtensionTypes, Link
from ..version import STAC_VERSION


class LandingPage(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#ogc-api---features-endpoints
    """

    description: str
    title: Optional[str]
    stac_version: str = Field(STAC_VERSION, const=True)
    stac_extensions: Optional[List[Union[str, ExtensionTypes]]]
    links: List[Link]



from typing import Any, Dict, List, Optional, Union

from geojson_pydantic.geometries import Polygon
from pydantic import BaseModel, Field

from ..shared import BBox, NumType


class CentroidObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/projection#centroid-object
    """

    lat: NumType
    lon: NumType


class ProjectionExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/projection#projection-extension-specification
    """

    epsg: Optional[Union[int]] = Field(..., alias="proj:epsg")
    wk2: Optional[Union[str, None]] = Field(None, alias="proj:wk2")
    projjson: Optional[Union[Dict[Any, Any], None]] = Field(None, alias="proj:projjson")
    geometry: Optional[Polygon] = Field(None, alias="proj:geometry")
    bbox: Optional[BBox] = Field(None, alias="proj:bbox")
    centroid: Optional[CentroidObject] = Field(None, alias="proj:centroid")
    shape: Optional[List[int]] = Field(None, alias="proj:shape")
    transform: Optional[List[float]] = Field(None, alias="proj:transform")

    class Config:
        allow_population_by_field_name = True



from enum import auto

from pydantic import BaseModel

from ...utils import AutoValueEnum


class SortDirections(str, AutoValueEnum):
    asc = auto()
    desc = auto()


class SortExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/tree/master/extensions/sort#sort-api-extension
    """

    field: str
    direction: SortDirections



from typing import List, Optional

from pydantic import BaseModel, Field


class PublicationObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/scientific#publication-object
    """

    doi: Optional[str]
    citation: Optional[str]


class ScientificExtension(PublicationObject):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/scientific#item-and-collection-fields
    """

    doi: Optional[str] = Field(None, alias="sci:doi")
    publications: Optional[List[PublicationObject]] = Field(
        None, alias="sci:publications"
    )
    citation: Optional[str] = Field(None, alias="sci:citation")

    class Config:
        allow_population_by_field_name = True



from typing import List

from pydantic import AnyHttpUrl, BaseModel


class ConformanceClasses(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#ogc-api---features-endpoints
    """

    conformsTo: List[AnyHttpUrl]



from typing import List

from pydantic import BaseModel

from ..shared import Asset


class CollectionAssetExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/collection-assets
    """

    assets: List[Asset]



from datetime import datetime
from typing import Optional, Union

from pydantic import BaseModel, Field


class TimestampsExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/timestamps
    """

    published: Optional[Union[str, datetime]] = Field(None, alias="published")
    expires: Optional[Union[str, datetime]] = Field(None, alias="expires")
    unpublished: Optional[Union[str, datetime]] = Field(None, alias="unpublished")



from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from .catalog import Catalog
from .shared import NumType, Provider


class SpatialExtent(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#spatial-extent-object
    """

    bbox: List[List[NumType]]


class TimeInterval(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#temporal-extent-object
    """

    interval: List[List[Union[str, None]]]


class Extent(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#extent-object
    """

    spatial: SpatialExtent
    temporal: TimeInterval


class Stats(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#stats-object
    """

    min: Union[NumType, str]
    max: Union[NumType, str]


class Collection(Catalog):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md
    """

    license: str
    extent: Extent
    title: Optional[str]
    keywords: Optional[List[str]]
    providers: Optional[List[Provider]]
    summaries: Optional[Dict[str, Union[Stats, List[Any]]]]



import click
import requests
from pydantic import ValidationError

from stac_pydantic import validate_item as validate


@click.group(short_help="Validate STAC")
def app():
    """stac-pydantic cli group"""
    pass


@app.command(short_help="Validate STAC Item")
@click.argument("infile")
def validate_item(infile):
    """Validate stac item"""
    r = requests.get(infile)
    r.raise_for_status()
    stac_item = r.json()
    try:
        validate(stac_item, skip_remote_refs=True, reraise_exception=True)
    except ValidationError as e:
        click.echo(str(e))
        return
    click.echo(f"{infile} is valid")
    return



from enum import auto
from typing import List, Optional

from pydantic import BaseModel, Field

from ..shared import NumType
from ..utils import AutoValueEnum


class PolarizationEnum(str, AutoValueEnum):
    HH = auto()
    VV = auto()
    HV = auto()
    VH = auto()


class Polarizations(BaseModel):
    __root__: List[PolarizationEnum]

    def __getitem__(self, item):
        return self.__root__[item].value


class FrequencyBands(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/sar#common-frequency-band-names
    """

    P = auto()
    L = auto()
    S = auto()
    C = auto()
    X = auto()
    Ku = auto()
    K = auto()
    Ka = auto()


class ObservationDirections(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/sar#item-fields
    """

    left = auto()
    right = auto()


class SARExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/sar#sar-extension-specification
    """

    instrument_mode: str = Field(..., alias="sar:instrument_mode")
    center_frequency: Optional[NumType] = Field(None, alias="sar:center_frequency")
    polarizations: Polarizations = Field(..., alias="sar:polarizations")
    product_type: str = Field(..., alias="sar:product_type")
    resolution_range: Optional[int] = Field(None, alias="sar:resolution_range")
    resolution_azimuth: Optional[int] = Field(None, alias="sar:resolution_azimuth")
    pixel_spacing_range: Optional[int] = Field(None, alias="sar:pixel_spacing_range")
    pixel_spacing_azimuth: Optional[int] = Field(
        None, alias="sar:pixel_spacing_azimuth"
    )
    looks_range: Optional[int] = Field(None, alias="sar:look_range")
    looks_azimuth: Optional[NumType] = Field(None, alias="sar:looks_azimuth")
    looks_equivalent_number: Optional[NumType] = Field(
        None, alias="sar:looks_equivalent_number"
    )
    observation_direction: Optional[ObservationDirections] = Field(
        None, alias="sar:observation_direction"
    )
    frequency_band: FrequencyBands = Field(..., alias="sar:frequency_band")

    class Config:
        allow_population_by_field_name = True
        use_enum_values = True



from datetime import datetime
from typing import Any, Dict, List, Optional, Union

from geojson_pydantic.geometries import (
    LineString,
    MultiLineString,
    MultiPoint,
    MultiPolygon,
    Point,
    Polygon,
)
from pydantic import BaseModel, Field, validator

from ..shared import DATETIME_RFC339, BBox
from .extensions.fields import FieldsExtension
from .extensions.query import Operator
from .extensions.sort import SortExtension


class Search(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/blob/master/api-spec.md#filter-parameters-and-fields
    """

    collections: List[str]
    ids: Optional[List[str]]
    bbox: Optional[BBox]
    intersects: Optional[
        Union[Point, MultiPoint, LineString, MultiLineString, Polygon, MultiPolygon]
    ]
    datetime: Optional[str]
    limit: int = 10
    field: Optional[FieldsExtension] = Field(None, alias="fields")
    query: Optional[Dict[str, Dict[Operator, Any]]]
    sortby: Optional[List[SortExtension]]

    @validator("intersects")
    def validate_spatial(cls, v, values):
        if v and values["bbox"]:
            raise ValueError("intersects and bbox parameters are mutually exclusive")
        return v

    @validator("datetime")
    def validate_datetime(cls, v):
        if "/" in v:
            values = v.split("/")
        else:
            # Single date is interpreted as end date
            values = ["..", v]

        dates = []
        for value in values:
            if value == "..":
                dates.append(value)
                continue
            try:
                datetime.strptime(value, DATETIME_RFC339)
                dates.append(value)
            except:
                raise ValueError(
                    f"Invalid datetime, must match format ({DATETIME_RFC339})."
                )

        if ".." not in dates:
            if datetime.strptime(dates[0], DATETIME_RFC339) > datetime.strptime(
                dates[1], DATETIME_RFC339
            ):
                raise ValueError(
                    "Invalid datetime range, must match format (begin_date, end_date)"
                )

        return dates



from .catalog import Catalog
from .collection import Collection
from .extensions import Extensions
from .item import (
    Item,
    ItemCollection,
    ItemProperties,
    item_model_factory,
    validate_item,
)



from setuptools import find_packages, setup

with open("README.md") as f:
    desc = f.read()

setup(
    name="stac-pydantic",
    description="Pydantic data models for the STAC spec",
    long_description=desc,
    long_description_content_type="text/markdown",
    version="1.3.7",
    python_requires=">=3.6",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
    ],
    keywords="stac pydantic validation",
    author=u"Arturo Engineering",
    author_email="engineering@arturo.ai",
    url="https://github.com/arturo-ai/stac-pydantic",
    license="MIT",
    packages=find_packages(exclude=["tests"]),
    zip_safe=False,
    install_requires=["click", "pydantic>=1.6", "geojson-pydantic",],
    tests_require=["pytest", "pytest-cov", "requests", "shapely"],
    setup_requires=["pytest-runner"],
    entry_points={"console_scripts": ["stac-pydantic=stac_pydantic.scripts.cli:app"]},
    package_data={"stac_pydantic": ["*.typed"]},
)



from enum import auto

from ...utils import AutoValueEnum


class Operator(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-api-spec/tree/master/extensions/query#query-api-extension
    """

    eq = auto()
    ne = auto()
    lt = auto()
    le = auto()
    gt = auto()
    ge = auto()
    startsWith = auto()
    endsWith = auto()
    constains = auto()



from typing import List, Optional

from pydantic import BaseModel, Field, root_validator

from .extensions import Extensions
from .shared import Link
from .version import STAC_VERSION


class Catalog(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/catalog-spec/catalog-spec.md
    """

    id: str
    description: str
    stac_version: str = Field(STAC_VERSION, const=True)
    links: List[Link]
    stac_extensions: Optional[List[str]]
    title: Optional[str]

    class Config:
        use_enum_values = True
        extra = "allow"

    @root_validator(pre=True)
    def validate_extensions(cls, values):
        if "stac_extensions" in values:
            for ext in values["stac_extensions"]:
                if ext in ("collection-assets", "item-assets", "version"):
                    ext_model = Extensions.get(ext)
                    ext_model(**values)
        return values

    def to_dict(self, **kwargs):
        return self.dict(by_alias=True, exclude_unset=True, **kwargs)

    def to_json(self, **kwargs):
        return self.json(by_alias=True, exclude_unset=True, **kwargs)



from datetime import datetime
from enum import Enum, auto
from typing import List, Optional, Tuple, Union

from pydantic import BaseModel, Extra, Field

from .extensions.eo import BandObject
from .utils import AutoValueEnum

NumType = Union[float, int]
BBox = Union[
    Tuple[NumType, NumType, NumType, NumType],  # 2D bbox
    Tuple[NumType, NumType, NumType, NumType, NumType, NumType],  # 3D bbox
]

# https://tools.ietf.org/html/rfc3339#section-5.6
DATETIME_RFC339 = "%Y-%m-%dT%H:%M:%SZ"


class ExtensionTypes(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/extensions/README.md#list-of-content-extensions
    """

    asset = auto()
    checksum = auto()
    collection_assets = "collection-assets"
    context = auto()
    cube = auto()
    eo = auto()
    item_assets = "item-assets"
    label = auto()
    pc = auto()
    projection = auto()
    sar = auto()
    sat = auto()
    sci = auto()
    single_file_stac = "single-file-stac"
    version = auto()
    view = auto()


class MimeTypes(str, Enum):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/item-spec.md#media-types
    """

    # Raster
    geotiff = "image/tiff; application=geotiff"
    cog = "image/tiff; application=geotiff; profile=cloud-optimized"
    jp2 = "image/jp2"
    png = "image/png"
    jpeg = "image/jpeg"
    # Vector
    geojson = "application/geo+json"
    geopackage = "application/geopackage+sqlite3"
    kml = "application/vnd.google-earth.kml+xml"
    kmz = "application/vnd.google-earth.kmz"
    # Others
    hdf = "application/x-hdf"
    hdf5 = "application/x-hdf5"
    xml = "application/xml"
    json = "application/json"
    html = "text/html"
    text = "text/plain"


class Relations(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#relation-types
    """

    self = auto()
    root = auto()
    parent = auto()
    child = auto()
    item = auto()
    license = auto()
    derived_from = auto()
    collection = auto()
    alternate = auto()
    previous = auto()
    next = auto()
    conformance = auto()
    docs = auto()
    tiles = auto()
    search = auto()


class AssetRoles(str, AutoValueEnum):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/extensions/asset/README.md
    """

    thumbnail = auto()
    overview = auto()
    data = auto()
    metadata = auto()


class ProviderRoles(str, AutoValueEnum):
    licensor = auto()
    producer = auto()
    processor = auto()
    host = auto()


class Link(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#link-object
    """

    href: str
    rel: str
    type: Optional[str]
    title: Optional[str]
    # Label extension
    label: Optional[str] = Field(None, alias="label:assets")

    class Config:
        use_enum_values = True


class Provider(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/collection-spec/collection-spec.md#provider-object
    """

    name: str
    description: Optional[str]
    roles: Optional[List[str]]
    url: Optional[str]


class StacCommonMetadata(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/common-metadata.md#date-and-time-range
    """

    title: Optional[str] = Field(None, alias="title")
    description: Optional[str] = Field(None, alias="description")
    start_datetime: Optional[Union[str, datetime]] = Field(None, alias="start_datetime")
    end_datetime: Optional[Union[str, datetime]] = Field(None, alias="end_datetime")
    created: Optional[Union[str, datetime]] = Field(None, alias="created")
    updated: Optional[Union[str, datetime]] = Field(None, alias="updated")
    platform: Optional[str] = Field(None, alias="platform")
    instruments: Optional[List[str]] = Field(None, alias="instruments")
    constellation: Optional[str] = Field(None, alias="constellation")
    mission: Optional[str] = Field(None, alias="mission")
    providers: Optional[List[Provider]] = Field(None, alias="providers")
    gsd: Optional[NumType] = Field(None, alias="gsd")


class Asset(StacCommonMetadata):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/item-spec.md#asset-object
    """

    href: str
    type: Optional[str]
    title: Optional[str]
    description: Optional[str]
    roles: Optional[List[AssetRoles]]
    # EO extension
    bands: Optional[List[BandObject]] = Field(None, alias="eo:bands")
    # SAR extension
    polarizations: Optional[List[str]] = Field(None, alias="sar:polarizations")
    # Checksum extension
    multihash: Optional[str] = Field(None, alias="checksum:multihash")

    class Config:
        allow_population_by_field_name = True
        use_enum_values = True
        extra = Extra.allow



from typing import Optional, Set

from pydantic import BaseModel


class FieldsExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/tree/master/extensions/fields#fields-api-extension
    """

    includes: Optional[Set[str]]
    excludes: Optional[Set[str]]



from datetime import datetime as dt
from functools import lru_cache
from typing import Dict, List, Optional, Tuple, Type, Union

from geojson_pydantic.features import Feature, FeatureCollection
from pydantic import BaseModel, Field, create_model, validator
from pydantic.fields import FieldInfo

from .api.extensions.context import ContextExtension
from .api.extensions.paging import PaginationLink
from .extensions import Extensions
from .shared import Asset, BBox, Link, StacCommonMetadata
from .utils import decompose_model
from .version import STAC_VERSION


class ItemProperties(StacCommonMetadata):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/item-spec.md#properties-object
    """

    datetime: Union[str, dt] = Field(..., alias="datetime")

    @validator("datetime")
    def validate_datetime(cls, v, values):
        if v == "null":
            if not values["start_datetime"] and not values["end_datetime"]:
                raise ValueError(
                    "start_datetime and end_datetime must be specified when datetime is null"
                )
        return v

    class Config:
        extra = "allow"


class Item(Feature):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/item-spec.md
    """

    id: str
    stac_version: str = Field(STAC_VERSION, const=True)
    properties: ItemProperties
    assets: Dict[str, Asset]
    links: List[Link]
    bbox: BBox
    stac_extensions: Optional[List[str]]
    collection: Optional[str]

    def to_dict(self, **kwargs):
        return self.dict(by_alias=True, exclude_unset=True, **kwargs)

    def to_json(self, **kwargs):
        return self.json(by_alias=True, exclude_unset=True, **kwargs)


class ItemCollection(FeatureCollection):
    """
    https://github.com/radiantearth/stac-spec/blob/v1.0.0-beta.1/item-spec/itemcollection-spec.md
    """

    stac_version: str = Field(STAC_VERSION, const=True)
    features: List[Item]
    stac_extensions: Optional[List[str]]
    links: List[Union[PaginationLink, Link]]
    context: Optional[ContextExtension]

    def to_dict(self, **kwargs):
        return self.dict(by_alias=True, exclude_unset=True, **kwargs)

    def to_json(self, **kwargs):
        return self.json(by_alias=True, exclude_unset=True, **kwargs)


@lru_cache()
def _extension_model_factory(
    stac_extensions: Tuple[str], base_class: Type[Item], skip_remote_refs: bool = False
) -> Tuple[Type[BaseModel], FieldInfo]:
    """
    Create a stac item properties model for a set of stac extensions
    """
    fields = decompose_model(base_class.__fields__["properties"].type_)
    for ext in stac_extensions:
        if skip_remote_refs and ext.startswith("http"):
            continue
        if ext == "checksum":
            continue
        fields.update(decompose_model(Extensions.get(ext)))
    return (
        create_model("CustomItemProperties", __base__=ItemProperties, **fields),
        FieldInfo(...),
    )


def item_model_factory(
    item: Dict, skip_remote_refs: bool = False, base_class: Type[Item] = Item
) -> Type[BaseModel]:
    """
    Create a pydantic model based on the extensions used by the item
    """
    item_fields = decompose_model(base_class)
    stac_extensions = item.get("stac_extensions")

    if stac_extensions:
        item_fields["properties"] = _extension_model_factory(
            tuple(stac_extensions), base_class, skip_remote_refs
        )

    return create_model("CustomStacItem", **item_fields, __base__=base_class)


def validate_item(item: Dict, reraise_exception: bool = False, **kwargs) -> bool:
    """
    Wrapper around ``item_model_factory`` for stac item validation
    """
    try:
        item_model_factory(item, **kwargs)(**item)
    except Exception:
        if reraise_exception:
            raise
        return False
    return True



from enum import auto
from typing import List, Optional, Union

from pydantic import BaseModel, Field

from ..shared import NumType
from ..utils import AutoValueEnum


class LabelTypes(str, AutoValueEnum):
    vector = auto()
    raster = auto()


class ClassObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/label#class-object
    """

    name: Optional[Union[str]]
    classes: Optional[List[Union[str, int]]]


class CountObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/label#count-object
    """

    name: Optional[str]
    count: Optional[int]


class StatsObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/label#stats-object
    """

    name: Optional[str]
    value: Optional[NumType]


class OverviewObject(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/label#label-overview-object
    """

    property_key: Optional[str]
    counts: Optional[List[CountObject]]
    statistics: Optional[List[StatsObject]]


class LabelExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-spec/tree/v1.0.0-beta.1/extensions/label#label-extension-specification
    """

    properties: List[Union[str, None]] = Field(..., alias="label:properties")
    classes: List[ClassObject] = Field(..., alias="label:classes")
    description: str = Field(..., alias="label:description")
    type: LabelTypes = Field(..., alias="label:type")
    tasks: Optional[List[str]] = Field(None, alias="label:tasks")
    methods: Optional[List[str]] = Field(None, alias="label:methods")
    overviews: Optional[List[OverviewObject]] = Field(None, alias="label:overviews")

    class Config:
        use_enum_values = True
        allow_population_by_field_name = True



from typing import Optional

from pydantic import BaseModel, validator


class ContextExtension(BaseModel):
    """
    https://github.com/radiantearth/stac-api-spec/tree/master/extensions/context#context-extension-specification
    """

    returned: int
    limit: Optional[int]
    matched: Optional[int]

    @validator("limit")
    def validate_limit(cls, v, values):
        if values["returned"] > v:
            raise ValueError(
                "Number of returned items must be less than or equal to the limit"
            )
